
/*****************************************************************************
 * GWCI    GNOME Web CGI Interface                                           *
 * A GNOME panel applet that accepts entrybox input and appends the result   *
 * a launcher command. Does URL armoring if the given string, intended to    *
 * launcher web browsers to make web searches and other cgi activities a     *
 * little easier.                                                            *
 *                                                                           *
 * Copyright (C) 2000 Erik Greenwald <erik@smluc.org>                        *
 * This program is free software; you can redistribute it and/or modify      * 
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <fcntl.h>
#include <unistd.h>
#include <gnome.h>
#include <applet-widget.h>
#include <tree.h>
#include <parser.h>
#include "about.h"
#include "conf.h"
#include "main.h"

#define NAME		0x01
#define TOOLTIP		0x02
#define LAUNCHER	0x03

	/*
	 * keep this global so we can use it in the config dialog 
	 */
int ei, gnamesig = 0, gtooltipsig = 0, glaunchersig = 0, changed = 0;
launcher **l, **tl;
GtkWidget *ctree, *gname, *gtooltip, *glauncher;

void
purge (launcher ** l)
{
    free (l[0]->name);
    free (l[0]->launcher);
    free (l[0]->tooltip);
    free (l[0]);
    l[0] = NULL;
}

void
set_editbox (GtkCTree * cctree, launcher ** tl)
{
    int i = 0;
    char *name[2];

    if (tl == NULL || tl[0] == NULL || cctree == NULL)
	return;
    gtk_clist_clear (&cctree->clist);
    while (tl[i] != NULL)
    {
	name[0] = tl[i]->name;
	name[1] = NULL;
	gtk_ctree_insert_node (cctree, NULL, NULL, name, 0, NULL,
			       NULL, NULL, NULL, FALSE, FALSE);
	i++;
    }
}

void
save_conf ()
{
    char buf[BUFSIZ];
    int f, i = 0;

    snprintf (buf, BUFSIZ, "%s/.gnome/gwci.xml", getenv ("HOME"));
    f = open (buf, O_WRONLY | O_CREAT | O_TRUNC);
    snprintf (buf, 1024, "<?xml version=\"1.0\"?>\n\n<GWCI-config>\n\n");
    write (f, buf, strlen (buf));
    while (l[i] != NULL)
    {
	snprintf (buf, 1024,
		  "<Launcher>\n\t<Name>%s</Name>\n\t<Tooltip>%s</Tooltip>\n\t<Prog>%s</Prog>\n</Launcher>\n\n",
		  l[i]->name, l[i]->tooltip, l[i]->launcher);
	write (f, buf, strlen (buf));
	i++;
    }
    snprintf (buf, 1024, "</GWCI-config>\n");
    write (f, buf, strlen (buf));
    close (f);
    return;
}

void
disconnect ()
{
    if (gnamesig != 0)
    {
	gtk_signal_disconnect (GTK_OBJECT (gname), gnamesig);
	gnamesig = 0;
	gtk_signal_disconnect (GTK_OBJECT (gtooltip), gtooltipsig);
	gtooltipsig = 0;
	gtk_signal_disconnect (GTK_OBJECT (glauncher), glaunchersig);
	glaunchersig = 0;
    }
}

void
clear_list (launcher ** ll)
{
    int i = 0;

    if (ll == NULL)
	return;
    while (ll[i] != NULL)
	purge (&ll[i++]);
    if (ll == l)
	l = NULL;
    if (ll == tl)
	tl = NULL;
    free (ll);
    return;
}

void
clear_temps ()
{
    clear_list (tl);
}

launcher **
dup_list (launcher ** l)
{
    int i = 0;
    launcher **ll;

    if (l == NULL)
	return NULL;
    ll = (launcher **) malloc (sizeof (launcher) * 100);
    memset (ll, 0, sizeof (launcher) * 100);
    while (l[i] != NULL)
    {
	ll[i] = (launcher *) malloc (sizeof (launcher));
	ll[i]->name = strdup (l[i]->name);
	ll[i]->tooltip = strdup (l[i]->tooltip);
	ll[i]->launcher = strdup (l[i]->launcher);
	i++;
    }
    return ll;
}

void
apply_changes ()
{
    int i;

    if (!changed)
	return;
    disconnect ();
    clear_list (l);
    l = dup_list (tl);
    gtk_widget_destroy (vbox);
    make_applet_body (tl);
    applet_widget_add (APPLET_WIDGET (applet), vbox);
    gtk_widget_show_all (applet);
    set_editbox (GTK_CTREE (ctree), l);
    save_conf ();
    changed = 0;
    return;
}

static void
rec_read (xmlNode * n, launcher ** cl, launcher * c, int x)
{
    int i;

    if (strcmp (n->name, "text"))
    {
	if (!strncmp (n->name, "Name", 5))
	    c->name = strdup (n->children->content);
	else if (!strncmp (n->name, "Tooltip", 8))
	    c->tooltip = strdup (n->children->content);
	else if (!strncmp (n->name, "Prog", 5))
	    c->launcher = strdup (n->children->content);
	else
	{
	    if (!strncmp (n->name, "Launcher", 9))
		c = cl[x++] = (launcher *) malloc (sizeof (launcher *));
	    if (n->children)
		rec_read (n->children, cl, c, x);
	}
    }
    if (n->next)
	rec_read (n->next, cl, c, x);
    return;
}

launcher **
load_conf (char *file)
{
    int i;
    xmlDocPtr doc;

    doc = xmlParseFile (file);
    if (doc == NULL)
	return NULL;
    l = (launcher **) malloc (sizeof (launcher *) * 100);
    memset (l, 0, sizeof (launcher *) * 100);
    rec_read (doc->children, l, NULL, 0);
    return l;
}

void
entry_changed (GtkEditable * e, gpointer d)
{
    gnome_property_box_changed (GNOME_PROPERTY_BOX
				(gtk_widget_get_toplevel (GTK_WIDGET (e))));
    changed = 1;
    switch ((int) d)
    {
    case NAME:
	free (tl[ei]->name);
	tl[ei]->name = strdup (gtk_entry_get_text (GTK_ENTRY (e)));
	set_editbox (GTK_CTREE (ctree), tl);
	break;
    case TOOLTIP:
	free (tl[ei]->tooltip);
	tl[ei]->tooltip = strdup (gtk_entry_get_text (GTK_ENTRY (e)));
	set_editbox (GTK_CTREE (ctree), tl);
	break;
    case LAUNCHER:
	free (tl[ei]->launcher);
	tl[ei]->launcher = strdup (gtk_entry_get_text (GTK_ENTRY (e)));
	set_editbox (GTK_CTREE (ctree), tl);
	break;
    default:
	printf ("Critical error?\n");
    }
    return;
}

void
new_launcher (GtkObject * w, gpointer d)
{
    int i = 0;
    char *name[2];

    while (tl[i] != NULL)
	i++;
    tl[i] = (launcher *) malloc (sizeof (launcher));
    tl[i]->name = strdup ("New");
    tl[i]->tooltip = strdup ("New Launcher");
    tl[i]->launcher = strdup ("New Command");
    name[0] = tl[i]->name;
    name[1] = NULL;
    i++;
    changed = 1;
    gnome_property_box_changed (GNOME_PROPERTY_BOX
				(gtk_widget_get_toplevel (GTK_WIDGET (w))));
    set_editbox (GTK_CTREE (ctree), tl);
}

void
delete_launcher (GtkObject * w, gpointer d)
{
    int i = (&(GTK_CTREE (ctree)->clist))->focus_row;
    GtkCTreeNode *n;

    disconnect ();
    purge (&tl[i]);
    tl[i] = (launcher *) 1;
    while (tl[i] != NULL)
    {
	tl[i] = tl[i + 1];
	i++;
    }
    tl[i] = NULL;
    tl[i + 1] = NULL;
    gnome_property_box_changed (GNOME_PROPERTY_BOX
				(gtk_widget_get_toplevel (GTK_WIDGET (w))));
    set_editbox (GTK_CTREE (ctree), tl);
    changed = 1;
    return;
}


void
on_ctree_tree_select_row (GtkCTree * ctree, GList * node, gint column,
			  gpointer user_data)
{
    ei = (&(ctree->clist))->focus_row;
    disconnect ();
    gtk_entry_set_text (GTK_ENTRY (gname), tl[ei]->name);
    gtk_entry_set_text (GTK_ENTRY (gtooltip), tl[ei]->tooltip);
    gtk_entry_set_text (GTK_ENTRY (glauncher), tl[ei]->launcher);
    gnamesig =
	gtk_signal_connect (GTK_OBJECT (gname), "changed",
			    GTK_SIGNAL_FUNC (entry_changed), (gpointer) NAME);
    gtooltipsig =
	gtk_signal_connect (GTK_OBJECT (gtooltip), "changed",
			    GTK_SIGNAL_FUNC (entry_changed),
			    (gpointer) TOOLTIP);
    glaunchersig =
	gtk_signal_connect (GTK_OBJECT (glauncher), "changed",
			    GTK_SIGNAL_FUNC (entry_changed),
			    (gpointer) LAUNCHER);
    return;
}

void
conf (void)
{				/* * static so if we click it again, another
				 * won't pop up. */
    static GtkWidget *dialog = NULL, *button;
    char *name[2];
    int i = 0;
    GtkWidget *hpaned, *label, *entry, *table, *scroll, *vbox2, *hbox;

    if (dialog)
    {
	gtk_widget_show_all (dialog);
	gdk_window_raise (dialog->window);
	return;
    }
    dialog = gnome_property_box_new ();
    clear_list (tl);
    tl = dup_list (l);
    hpaned = gtk_hpaned_new ();
    gtk_paned_set_gutter_size (GTK_PANED (hpaned), 16);
    gtk_paned_set_handle_size (GTK_PANED (hpaned), 6);
    gtk_paned_set_position (GTK_PANED (hpaned), 120);
    scroll = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
    ctree = gtk_ctree_new (1, 0);
    gtk_signal_connect (GTK_OBJECT (ctree), "tree_select_row",
			GTK_SIGNAL_FUNC (on_ctree_tree_select_row), NULL);
    gtk_container_add (GTK_CONTAINER (scroll), ctree);
    gtk_clist_column_titles_show (GTK_CLIST (ctree));
    gtk_clist_set_column_widget (GTK_CLIST (ctree), 0,
				 gtk_label_new ("Button"));
    hbox = gtk_hbox_new (TRUE, 4);
    vbox2 = gtk_vbox_new (FALSE, 4);
    gtk_paned_add1 (GTK_PANED (hpaned), scroll);
    table = gtk_table_new (3, 2, FALSE);
    gtk_table_set_row_spacings (GTK_TABLE (table), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table), 4);
    gname = gtk_entry_new ();
    label = gtk_label_new ("Name");
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		      (GtkAttachOptions) GTK_FILL, 0, 0, 0);
    gtk_table_attach (GTK_TABLE (table), gname, 1, 2, 0, 1,
		      (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 0, 0, 0);
    gtooltip = gtk_entry_new ();
    label = gtk_label_new ("Tooltip");
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
		      (GtkAttachOptions) GTK_FILL, 0, 0, 0);
    gtk_table_attach (GTK_TABLE (table), gtooltip, 1, 2, 1, 2,
		      (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 0, 0, 0);
    glauncher = gtk_entry_new ();
    label = gtk_label_new ("Launcher");
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
		      (GtkAttachOptions) GTK_FILL, 0, 0, 0);
    gtk_table_attach (GTK_TABLE (table), glauncher, 1, 2, 2, 3,
		      (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 0, 0, 0);
    button = gtk_button_new_with_label ("New");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (new_launcher), NULL);
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, FALSE, 0);
    button = gtk_button_new_with_label ("Delete");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (delete_launcher), NULL);
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox2), hbox, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox2), table, TRUE, TRUE, 0);
    gtk_paned_pack2 (GTK_PANED (hpaned), vbox2, TRUE, TRUE);
    gnome_property_box_append_page (GNOME_PROPERTY_BOX (dialog), hpaned,
				    gtk_label_new _("Buttons"));
    gtk_window_set_policy (GTK_WINDOW (dialog), TRUE, TRUE, FALSE);
    gtk_widget_set (dialog, "title", _("GWCI Settings"),
		    "signal::apply", apply_changes, NULL,
		    "signal::destroy", clear_temps, NULL,
		    "signal::destroy", gtk_widget_destroyed, &dialog,
		    "signal::help", help, NULL, NULL);
    gtk_quit_add_destroy (1, GTK_OBJECT (dialog));
    gtk_widget_show_all (dialog);
    gdk_window_raise (dialog->window);
    set_editbox (GTK_CTREE (ctree), l);
    return;
}
